@@ -1,7 +1,7 @@
 /****************************************************************************
   This file is part of the Freedom Remailer.  It is:
-  Copyright (C) 1995  John B. Fleming (jfleming@indiana.edu)
-  Changes are (C) 1997 Johannes Kroeger (hanne@squirrel.owl.de)
+  Copyright (C) 1995-1997  John B. Fleming (jfleming@indiana.edu)
+  Changes are (C) 1997-1998  Johannes Kroeger (hanne@squirrel.owl.de)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -48,10 +48,11 @@ char SENDMAIL[BUFSIZ] = "/usr/sbin/sendmail -t";
 /* the -h option tells inews to expect message headers in stdin */
 char INEWS[BUFSIZ] = "/usr/bin/inews -h";
 char PGP[BUFSIZ] = "/usr/bin/pgp";
-char LYNX[BUFSIZ] = "/usr/bin/lynx";
+char WGET[BUFSIZ] = "/usr/bin/wget";
 char HELP_FILE[BUFSIZ] = "freedom-help";
-char KEY_FILE[BUFSIZ] = "freedom-key";
 char STATS_FILE[BUFSIZ] = "freedom-stats";
+char KEY_FILE[BUFSIZ] = "freedom-key";
+char PASS_FILE[BUFSIZ] = "";
 char SOURCE_BLOCK[BUFSIZ] = "blocked.source";
 char DEST_BLOCK[BUFSIZ] = "blocked.destination";
 char SUBJ_BLOCK[BUFSIZ] = "blocked.subject";
@@ -61,7 +62,7 @@ char HDRFILTER[BUFSIZ] = "headers.del";
 char REMAILER_NAME[BUFSIZ] = "Anonymous Remailer";
 char ANON_NAME[BUFSIZ] = "Anonymous";
 char ORGANIZATION[BUFSIZ] = "Anonymous Posting Service";
-char MAIL2NEWS[BUFSIZ] = "mail2news_nospam@nym.alias.net";
+char MAIL2NEWS[BUFSIZ] = "mail2news@nym.alias.net";
 char REMAILER_ADDR[BUFSIZ] = "";
 char ANON_ADDR[BUFSIZ] = "";
 char COMPLAINTS[BUFSIZ] = "";
@@ -92,7 +93,7 @@ init_remailer(void)
   FILE *infile;
   char line[BUFSIZ], junk[BUFSIZ], *foo;
 
-  umask(007);
+  umask(077);
   remailer_dir[0] = '\0';
   strncat(remailer_dir, REMAILER_DIR, sizeof(remailer_dir) - 1);
 
@@ -155,10 +156,11 @@ init_remailer(void)
 	read_conf_w(SENDMAIL);
 	read_conf_w(INEWS);
 	read_conf(PGP);
-	read_conf(LYNX);
+	read_conf(WGET);
 	read_conf(HELP_FILE);
-	read_conf(KEY_FILE);
 	read_conf(STATS_FILE);
+	read_conf(KEY_FILE);
+	read_conf(PASS_FILE);
 	read_conf(SOURCE_BLOCK);
 	read_conf(DEST_BLOCK);
 	read_conf(SUBJ_BLOCK);
@@ -176,8 +178,12 @@ init_remailer(void)
     }
     if (ALLOW_LTIME && (QUEUE_DIR[0] == '\0'))
       strcpy(QUEUE_DIR, REMAILER_DIR "/queue");
-    if (ALLOW_PGP &&(PGP_DIR[0] == '\0'))
-      strcpy(PGP_DIR, REMAILER_DIR "/.pgp");
+    if (ALLOW_PGP) {
+      if (PGP_DIR[0] == '\0')
+	strcpy(PGP_DIR, REMAILER_DIR "/.pgp");
+      if (PASS_FILE[0] == '\0')
+	strcpy(PASS_FILE, REMAILER_DIR "/.pgp/passphrase");
+    }
     if (ANON_NAME[0] == '\0')
       strcpy(ANON_NAME, REMAILER_NAME);
     if (ANON_ADDR[0] == '\0')
@@ -216,11 +222,11 @@ main(int argc, char *argv[])
       exit(0);
     }
     else {
-      printf("Freedom Remailer 2.1.2 "
-	     "(C) John B. Fleming 1995, Johannes Kroeger 1997\n\n"
+      printf("Freedom Remailer %s\n"
+	     "(C) John B. Fleming 1995-1997, Johannes Kroeger 1997-1998\n\n"
 	     "Usage: %s\t\t# read mail on stdin and process\n"
 	     "       %s -q\t# run queue of latent messages\n",
-	     basename(argv[0]), basename(argv[0]));
+	     VERSION, basename(argv[0]), basename(argv[0]));
       exit(1);
     }
   }
@@ -264,16 +270,23 @@ main(int argc, char *argv[])
 
   recursing = 1;
   while (recursing) {
+    anon = scan_message(tempfilename, send_to, message_from, recursing);
     recursing = 0;
-    anon = scan_message(tempfilename, send_to, message_from);
-    if (anon == NULL_MESSAGE) {
+
+    switch (anon) {
+    case BLOCKED_SENDER:
+      if (USE_SYSLOG)
+	syslog(LOG_NOTICE, "Discarding blocked message");
+      unlink(tempfilename);
+    break;
+
+    case NULL_MESSAGE:
       if (USE_SYSLOG)
 	syslog(LOG_DEBUG, "Discarding dummy message");
       unlink(tempfilename);
-    }
-    else if (anon == PGP_MESSAGE) {
-      if (USE_STATS)
-	updatestats(STATS_PGP);
+    break;
+
+    case PGP_MESSAGE:
       infile = fopen(tempfilename, "r");
       /* extract first PGP packet of incoming message for decryption */
       if ((outfile = fopen(eFilename, "w"))) {
@@ -311,203 +324,24 @@ main(int argc, char *argv[])
       unlink(eFilename);
       unlink(dFilename);
       recursing = 1;
-    }
-    else if (anon == ANON_MESSAGE) {
+    break;
+
+    case ANON_MESSAGE:
       if (USE_SYSLOG)
 	syslog(LOG_INFO, "Anonymous message detected");
-      if (USE_STATS) /* do stats whether or not blocked */
-	updatestats(STATS_MESSAGE);
-      if (blocked(message_from, SOURCE_BLOCK)) {
+      remove_headers(tempfilename, bFilename, send_to);
+      if (scan_subject(bFilename) == -1) {
 	if (USE_SYSLOG)
 	  syslog(LOG_NOTICE, "Discarding blocked message");
-	unlink(tempfilename);
-      }
-      else {
-	remove_headers(tempfilename, bFilename, send_to);
-	unlink(tempfilename);
-	if (scan_subject(bFilename) == -1) {
-	  if (USE_SYSLOG)
-	    syslog(LOG_NOTICE, "Discarding blocked message");
-	  unlink(bFilename);
-	}
-	else {
-	  if (USE_SYSLOG)
-	    syslog(LOG_INFO, "Remailing message");
-	  if (USE_MIX == 1) {
-	    do /* make sure we don't overwrite other messages */
-	      sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
-	    while (link(bFilename, poolfilename) < 0);
-	  }
-	  else {
-	    if (!USE_MIX)
-	      sprintf(line, "%s", SENDMAIL);
-	    else if (USE_MIX == 2)
-	      sprintf(line, "%s/%s", MIX_DIR, MIX_MAIL);
-	    p = popen(line, "w");
-	    if (!USE_MIX) {
-	      fprintf(p, "From: %s <%s>\n", ANON_NAME, ANON_ADDR);
-	      fprintf(p, DISCLAIMER);
-	    }
-	    if ((infile = fopen(bFilename, "r"))) {
-	      while (fgets(line, sizeof(line), infile))
-		fputs(line, p);
-	      fclose(infile);
-	    }
-	    pclose(p);
-	  }
-	  unlink(bFilename);
-	}
-      }
-    }
-    else if (anon == ANON_POST) {
-      if (USE_SYSLOG)
-	syslog(LOG_INFO, "Anonymous post detected");
-      if (USE_STATS)
-	updatestats(STATS_POST);
-      if (blocked(message_from, SOURCE_BLOCK)) {
-	if (USE_SYSLOG)
-	  syslog(LOG_NOTICE, "Discarding blocked message");
-	unlink(tempfilename);
-      }
-      else {
-	remove_headers(tempfilename, bFilename, send_to);
-	unlink(tempfilename);
-	if (scan_subject(bFilename) == -1) {
-	  if (USE_SYSLOG)
-	    syslog(LOG_NOTICE, "Discarding blocked message");
-	  unlink(bFilename);
-	}
-	else {
-	  if (USE_SYSLOG)
-	    syslog(LOG_INFO, "Posting message");
-	  if (USE_MIX == 1) {
-	    do /* make sure we don't overwrite other messages */
-	      sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
-	    while (link(bFilename, poolfilename) < 0);
-	  }
-	  else {
-	    if (!USE_MIX) {
-	      if (ALLOW_POST == 1)
-		sprintf(line, "%s", INEWS);
-	      else if (ALLOW_POST == 2)
-		sprintf(line, "%s", SENDMAIL);
-	    }
-	    else if (USE_MIX == 2)
-	      sprintf(line, "%s/%s", MIX_DIR, MIX_NEWS);
-	    p = popen(line, "w");
-	    if (!USE_MIX) {
-	      if (ALLOW_POST == 1)
-		fprintf(p, "Organization: %s\n", ORGANIZATION);
-	      else if (ALLOW_POST == 2)
-		fprintf(p, "To: %s\n", MAIL2NEWS);
-	      fprintf(p, "From: %s <%s>\n", ANON_NAME, ANON_ADDR);
-	      fprintf(p, DISCLAIMER);
-	      if (!scan_subject(bFilename))
-		fprintf(p, "Subject: none\n");
-	    }
-	    if ((infile = fopen(bFilename, "r"))) {
-	      while (fgets(line, sizeof(line), infile))
-		fputs(line, p);
-	      fclose(infile);
-	    }
-	    pclose(p);
-	  }
-	  unlink(bFilename);
-	}
-      }
-    }
-    else if (anon == STATS_REQ) {
-      if (USE_SYSLOG)
-	syslog(LOG_INFO, "Statistics were requested");
-      updatestats(STATS_STAT);
-      if (blocked(message_from, DEST_BLOCK)) {
-	if (USE_SYSLOG)
-	  syslog(LOG_NOTICE, "Ignoring request");
-      }
-      else {
-	if (USE_SYSLOG)
-	  syslog(LOG_INFO, "Mailing statistics");
-	mailstats(message_from);
-      }
-      unlink(tempfilename);
-    }
-    else if (anon == HELP_REQ) {
-      if (USE_SYSLOG)
-	syslog(LOG_INFO, "Help was requested");
-      if (USE_STATS)
-	updatestats(STATS_HELP);
-      mailfile(HELP_FILE, message_from);
-      unlink(tempfilename);
-    }
-    else if (anon == KEY_REQ) {
-      if (USE_SYSLOG)
-	syslog(LOG_INFO, "PGP key was requested");
-      if (USE_STATS)
-	updatestats(STATS_KEY);
-      mailfile(KEY_FILE, message_from);
-      unlink(tempfilename);
-    }
-    else if (anon == CUTMARKS) {
-      if (USE_SYSLOG)
-	syslog(LOG_DEBUG, "Cutmarks: %s detected", send_to);
-      cut(tempfilename, cFilename, send_to, "cutmarks:", 1);
-      trimmings = 1;
-    }
-    else if (anon == ENCRYPT_SUB) {
-      if (USE_SYSLOG)
-	syslog(LOG_DEBUG, "Encrypt-Subject: %s detected", send_to);
-      encrypt_sub(tempfilename, send_to);
-      recursing = 1;
-    }
-    else if (anon == ENCRYPT_KEY) {
-      if (USE_SYSLOG)
-	syslog(LOG_DEBUG, "Encrypt-Key: %s detected", send_to);
-      cut(tempfilename, kFilename, "**", "encrypt-key:", 0);
-      encrypt_key(kFilename, tempfilename, send_to);
-      recursing = 1;
-    }
-    else if (anon == LATENT_TIME) {
-      if (USE_SYSLOG)
-	syslog(LOG_DEBUG, "Latent-Time: %s detected", send_to);
-      if (USE_STATS)
-	updatestats(STATS_LATENT);
-      queuelatent(tempfilename, send_to);
-      unlink(tempfilename);
-    }
-    else if ((anon == URL_REQUEST) || (anon == HTML_REQUEST)) {
-      if (anon == URL_REQUEST) {
-	if (USE_SYSLOG)
-	  syslog(LOG_INFO, "WWW page requested");
-      }
-      else if (anon == HTML_REQUEST) {
-	if (USE_SYSLOG)
-	  syslog(LOG_INFO, "HTML source requested");
-      }
-      if (USE_STATS)
-	updatestats(STATS_WWW);
-      if (strlen(send_to) == 0) {
-	if (USE_SYSLOG)
-	  syslog(LOG_NOTICE, "Ignoring request");
 	unlink(bFilename);
       }
       else {
-	remove_headers(tempfilename, bFilename, send_to);
-	unlink(tempfilename);
-	geturl(message_from, uFilename, anon);
 	if (USE_SYSLOG)
 	  syslog(LOG_INFO, "Remailing message");
 	if (USE_MIX == 1) {
 	  do /* make sure we don't overwrite other messages */
 	    sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
 	  while (link(bFilename, poolfilename) < 0);
-	  outfile = fopen(poolfilename, "a");
-	  if ((infile = fopen(uFilename, "r"))) {
-	    fputs("\n", outfile);
-	    while (fgets(line, sizeof(line), infile))
-	      fputs(line, outfile);
-	    fclose(infile);
-	  }
-	  fclose(outfile);
 	}
 	else {
 	  if (!USE_MIX)
@@ -524,8 +358,50 @@ main(int argc, char *argv[])
 	      fputs(line, p);
 	    fclose(infile);
 	  }
-	  if ((infile = fopen(uFilename, "r"))) {
-	    fputs("\n", p);
+	  pclose(p);
+	}
+	unlink(bFilename);
+      }
+    break;
+
+    case ANON_POST:
+      if (USE_SYSLOG)
+	syslog(LOG_INFO, "Anonymous post detected");
+      remove_headers(tempfilename, bFilename, send_to);
+      if (scan_subject(bFilename) == -1) {
+	if (USE_SYSLOG)
+	  syslog(LOG_NOTICE, "Discarding blocked message");
+	unlink(bFilename);
+      }
+      else {
+	if (USE_SYSLOG)
+	  syslog(LOG_INFO, "Posting message");
+	if (USE_MIX == 1) {
+	  do /* make sure we don't overwrite other messages */
+	    sprintf(poolfilename, "%s/mail%08x", MIX_DIR, rand());
+	  while (link(bFilename, poolfilename) < 0);
+	}
+	else {
+	  if (!USE_MIX) {
+	    if (ALLOW_POST == 1)
+	      sprintf(line, "%s", INEWS);
+	    else if (ALLOW_POST == 2)
+	      sprintf(line, "%s", SENDMAIL);
+	  }
+	  else if (USE_MIX == 2)
+	    sprintf(line, "%s/%s", MIX_DIR, MIX_NEWS);
+	  p = popen(line, "w");
+	  if (!USE_MIX) {
+	    if (ALLOW_POST == 1)
+	      fprintf(p, "Organization: %s\n", ORGANIZATION);
+	    else if (ALLOW_POST == 2)
+	      fprintf(p, "To: %s\n", MAIL2NEWS);
+	    fprintf(p, "From: %s <%s>\n", ANON_NAME, ANON_ADDR);
+	    fprintf(p, DISCLAIMER);
+	    if (!scan_subject(bFilename))
+	      fprintf(p, "Subject: none\n");
+	  }
+	  if ((infile = fopen(bFilename, "r"))) {
 	    while (fgets(line, sizeof(line), infile))
 	      fputs(line, p);
 	    fclose(infile);
@@ -533,21 +409,103 @@ main(int argc, char *argv[])
 	  pclose(p);
 	}
 	unlink(bFilename);
-	unlink(uFilename);
       }
-    }
-    else if (trimmings) {
+    break;
+
+    case STATS_REQ:
       if (USE_SYSLOG)
-	syslog(LOG_DEBUG, "Processing next part of message");
-      rename (cFilename, tempfilename);
-      trimmings = 0;
+	syslog(LOG_INFO, "Statistics were requested");
+      if (blocked(message_from, DEST_BLOCK)) {
+	if (USE_SYSLOG)
+	  syslog(LOG_NOTICE, "Ignoring request");
+      }
+      else {
+	if (USE_SYSLOG)
+	  syslog(LOG_INFO, "Mailing statistics");
+	mailstats(message_from);
+      }
+      unlink(tempfilename);
+    break;
+
+    case CONF_REQ:
+      if (USE_SYSLOG)
+	syslog(LOG_INFO, "Abilities were requested");
+      if (blocked(message_from, DEST_BLOCK)) {
+	if (USE_SYSLOG)
+	  syslog(LOG_NOTICE, "Ignoring request");
+      }
+      else {
+	if (USE_SYSLOG)
+	  syslog(LOG_INFO, "Mailing abilities");
+	mailconf(message_from);
+      }
+      unlink(tempfilename);
+    break;
+
+    case HELP_REQ:
+      if (USE_SYSLOG)
+	syslog(LOG_INFO, "Help was requested");
+      mailfile(HELP_FILE, message_from);
+      unlink(tempfilename);
+    break;
+
+    case KEY_REQ:
+      if (USE_SYSLOG)
+	syslog(LOG_INFO, "PGP key was requested");
+      mailfile(KEY_FILE, message_from);
+      unlink(tempfilename);
+    break;
+
+    case CUTMARKS:
+      if (USE_SYSLOG)
+	syslog(LOG_DEBUG, "Cutmarks: %s detected", send_to);
+      cut(tempfilename, cFilename, send_to, "cutmarks:", 1);
+      trimmings = 1;
+    break;
+
+    case URL_REQUEST:
+      if (USE_SYSLOG)
+	syslog(LOG_INFO, "URL was requested");
+      append_url(uFilename, tempfilename, send_to);
+      recursing = 2;
+    break;
+
+    case ENCRYPT_SUB:
+      if (USE_SYSLOG)
+	syslog(LOG_DEBUG, "Encrypt-Subject: %s detected", send_to);
+      encrypt_sub(tempfilename, send_to);
       recursing = 1;
-    }
-    else {
+    break;
+
+    case ENCRYPT_KEY:
       if (USE_SYSLOG)
-	syslog(LOG_INFO, "Non-anonymous message detected");
-      if ((infile = fopen(tempfilename, "r"))) {
-	if (fgets(line, sizeof(line), infile)) {
+	syslog(LOG_DEBUG, "Encrypt-Key: %s detected", send_to);
+      cut(tempfilename, kFilename, "**", "encrypt-key:", 0);
+      encrypt_key(kFilename, tempfilename, send_to);
+      recursing = 1;
+    break;
+
+    case LATENT_TIME:
+      if (USE_SYSLOG)
+	syslog(LOG_DEBUG, "Latent-Time: %s detected", send_to);
+      queuelatent(tempfilename, send_to);
+      unlink(tempfilename);
+    break;
+
+    case NON_ANON:
+      if (trimmings) {
+	if (USE_SYSLOG)
+	  syslog(LOG_DEBUG, "Processing next part of message");
+	rename (cFilename, tempfilename);
+	trimmings = 0;
+	recursing = 1;
+      }
+      else {
+	if (USE_SYSLOG)
+	  syslog(LOG_INFO, "Non-anonymous message detected");
+	if ((infile = fopen(tempfilename, "r"))) {
+	  fgets(line, sizeof(line), infile);
+	  rewind(infile);
 	  if (!strcmp(line, "Cut-Recursive: Yes\n")) {
 	    if (USE_SYSLOG) {
 	      syslog(LOG_DEBUG, "It's cutmarks leftovers");
@@ -564,17 +522,18 @@ main(int argc, char *argv[])
 	    if (USE_SYSLOG)
 	      syslog(LOG_INFO, "Appending message to spool");
 	    if ((outfile = fopen(MAIL_SPOOL, "a"))) {
-	      do
+	      while (fgets(line, sizeof(line), infile))
 		fputs(line, outfile);
-	      while (fgets(line, sizeof(line), infile));
 	      fclose(outfile);
 	    }
 	  }
+	  fclose(infile);
 	}
-	fclose(infile);
+	unlink(tempfilename);
       }
-      unlink(tempfilename);
+    break;
     }
+
     if (trimmings)
       recursing = 1;
   }
