@@ -1,7 +1,7 @@
 /****************************************************************************
   This file is part of the Freedom Remailer.  It is:
-  Copyright (C) 1995  John B. Fleming (jfleming@indiana.edu)
-  Changes are (C) 1997 Johannes Kroeger (hanne@squirrel.owl.de)
+  Copyright (C) 1995-1997  John B. Fleming (jfleming@indiana.edu)
+  Changes are (C) 1997-1998  Johannes Kroeger (hanne@squirrel.owl.de)
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,16 +20,17 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
 #include <ctype.h>
 #include <syslog.h>
 #include "freedom.h"
 
 enum message_type
-scan_message(const char *infilename, char *to, char *from)
+scan_message(const char *infilename, char *to, char *from, const int pass)
 {
   int headers = 1, colhead = 0, linenum = 0, replyto = 0,
       ctflag = 0, esflag = 0, ekflag = 0, ltflag = 0,
-      anflag = 0, ptflag = 0, urlflag = 0, htmlflag = 0, rrflag = 0;
+      anflag = 0, ptflag = 0, urlflag = 0;
   FILE *infile;
   char line[BUFSIZ], tmpline[BUFSIZ], cutmarks[BUFSIZ], encryptsub[BUFSIZ],
        encryptkey[BUFSIZ], latency[BUFSIZ], url[BUFSIZ];
@@ -42,7 +43,7 @@ scan_message(const char *infilename, char *to, char *from)
     return NON_ANON;
   if (USE_SYSLOG)
     syslog(LOG_DEBUG, "Scanning message");
-  while (fgets(line, sizeof(line), infile)) {
+  while (fgets(line, sizeof(line), infile) && linenum < 2) {
     if (!headers) {
       if ((linenum > colhead) || (strlen(line) > 1))
 	linenum++;
@@ -56,14 +57,23 @@ scan_message(const char *infilename, char *to, char *from)
 	headers = 0;
       else {
 	if (linenum == 0) { /* real headers */
-	  if (strileft(line, "reply-to:")) {
+	  if (strileft(line, "from ") ||
+	      strileft(line, "sender:")) {
+	    if (blocked(line, SOURCE_BLOCK))
+	      return BLOCKED_SENDER;
+	  }
+	  else if (strileft(line, "reply-to:")) {
 	    strcpy(from, line + sizeof("reply-to:") - 1);
 	    chop(from);
 	    replyto = 1;
+	    if (blocked(from, SOURCE_BLOCK))
+	      return BLOCKED_SENDER;
 	  }
 	  else if (strileft(line, "from:") && !replyto) {
 	    strcpy(from, line + sizeof("from:") - 1);
 	    chop(from);
+	    if (blocked(from, SOURCE_BLOCK))
+	      return BLOCKED_SENDER;
 	  }
 	  else if (strileft(line, "subject:")) {
 	    strcpy(tmpline, line + sizeof("subject:") - 1);
@@ -74,6 +84,10 @@ scan_message(const char *infilename, char *to, char *from)
 	      return HELP_REQ;
 	    else if (strieq(tmpline, "freedom-help"))
 	      return HELP_REQ;
+	    else if (strieq(tmpline, "remailer-conf"))
+	      return CONF_REQ;
+	    else if (strieq(tmpline, "freedom-conf"))
+	      return CONF_REQ;
 	    else if (USE_STATS && strieq(tmpline, "remailer-stats"))
 	      return STATS_REQ;
 	    else if (USE_STATS && strieq(tmpline, "freedom-stats"))
@@ -84,11 +98,13 @@ scan_message(const char *infilename, char *to, char *from)
 	      return KEY_REQ;
 	  }
 	}
-	if (ALLOW_PGP && strileft(line, "encrypted: pgp")) {
-	  chop(line);
+	if (strileft(line, "encrypted:")) {
+	  strcpy(tmpline, line + sizeof("encrypted:") - 1);
+	  chop(tmpline);
 	  if (USE_SYSLOG)
-	    syslog(LOG_DEBUG, "%s detected", line);
-	  return PGP_MESSAGE;
+	    syslog(LOG_DEBUG, "%s encrypted message detected", tmpline);
+	  if (ALLOW_PGP && strieq(tmpline, "pgp"))
+	    return PGP_MESSAGE;
 	}
 	else if (strileft(line, "anon-to:")) {
 	  strcpy(tmpline, line + sizeof("anon-to:") - 1);
@@ -143,44 +159,32 @@ scan_message(const char *infilename, char *to, char *from)
 	else if (strileft(line, "cutmarks:")) {
 	  strcpy(cutmarks, line + sizeof("cutmarks:") - 1);
 	  chop(cutmarks);
-	  ctflag = 1;
+	  if (strlen(cutmarks) > 0)
+	    ctflag = 1;
 	}
 	else if (strileft(line, "encrypt-subject:")) {
 	  strcpy(encryptsub, line + sizeof("encrypt-subject:") - 1);
 	  chop(encryptsub);
-	  esflag = 1;
+	  if (strlen(encryptsub) > 0)
+	    esflag = 1;
 	}
 	else if (ALLOW_PGP && strileft(line, "encrypt-key:")) {
 	  strcpy(encryptkey, line + sizeof("encrypt-key:") - 1);
 	  chop(encryptkey);
-	  ekflag = 1;
+	  if (strlen(encryptkey) > 0)
+	    ekflag = 1;
 	}
 	else if (ALLOW_LTIME && strileft(line, "latent-time:")) {
 	  strcpy(latency, line + sizeof("latent-time:") - 1);
 	  chop(latency);
-	  ltflag = 1;
+	  if (strlen(latency) > 0)
+	    ltflag = 1;
 	}
-	/* The following directives are only evaluated in the ::'ed headers
-	   to make sure that everything below the next blank line will be
-	   remailed to the Return-To: address */
-	else if (ALLOW_WWW && strileft(line, "get-url:") && colhead) {
+	else if (ALLOW_PGP && ALLOW_WWW && strileft(line, "get-url:")) {
 	  strcpy(url, line + sizeof("get-url:") - 1);
 	  chop(url);
-	  urlflag = 1;
-	}
-	else if (ALLOW_WWW && strileft(line, "get-html-source:") && colhead) {
-	  strcpy(url, line + sizeof("get-html-source:") - 1);
-	  chop(url);
-	  htmlflag = 1;
-	}
-	else if (ALLOW_WWW && strileft(line, "return-to:") && colhead) {
-	  strcpy(tmpline, line + sizeof("return-to:") - 1);
-	  chop(tmpline);
-	  if (!blocked(tmpline, DEST_BLOCK))
-	    strcpy(to, TO), strcat(to, tmpline), strcat(to, "\n");
-	  else
-	    to[0] = '\0';
-	  rrflag = 1;
+	  if (strlen(url) > 0 && pass != 2)
+	    urlflag = 1;
 	}
 	/* The Null: directive for bit-bucket mail makes sense only
 	   in the ::'ed headers of encrypted cover traffic messages */
@@ -195,21 +199,7 @@ scan_message(const char *infilename, char *to, char *from)
   }
   fclose(infile);
 
-  if (urlflag || htmlflag) {
-    if (!rrflag) {
-      if (!blocked(from, DEST_BLOCK))
-	strcpy(to, TO), strcat(to, from), strcat(to, "\n");
-      else
-	to[0] = '\0';
-    }
-    strcpy(from, url);
-    if (urlflag)
-      return URL_REQUEST;
-    if (htmlflag)
-      return HTML_REQUEST;
-  }
-
-  /* process headers in order: cutmarks, encrypt-subject, encrypt-key,
+  /* process headers in order: cutmarks, get-url, encrypt-subject, encrypt-key,
      latent-time */
   /* if these don't exist, then remail or post the message if that is where
      it's headed; else, return NON_ANON to send it to the spool or bucket */
@@ -218,6 +208,10 @@ scan_message(const char *infilename, char *to, char *from)
     strcpy(to, cutmarks);
     return CUTMARKS;
   }
+  if (urlflag) {
+    strcpy(to, url);
+    return URL_REQUEST;
+  }
   if (esflag) {
     strcpy(to, encryptsub);
     return ENCRYPT_SUB;
@@ -244,18 +238,29 @@ remove_headers(const char *infilename, const char *outfilename, const char *to)
   FILE *infile, *outfile;
   int headers = 1, colhead = 0, addhead = 0, linenum = 0;
   char line[BUFSIZ], tmpline[BUFSIZ];
+  enum stats_flag flag;
 
+  outfile = fopen(outfilename, "w");
   if (!(infile = fopen(infilename, "r"))) {
-    outfile = fopen(outfilename, "w");
     fclose(outfile);
     return;
   }
-  outfile = fopen(outfilename, "w");
+  else {
+    fgets(line, sizeof(line), infile);
+    rewind(infile);
+    if (USE_STATS) {
+      if (strcmp(line, "Cut-Recursive: Yes\n"))
+	flag = STATS_MESSAGE;
+      if (!strcmp(line, "PGP-Recursive: Yes\n"))
+	flag = STATS_PGP;
+      updatestats(flag);
+    }
+  }
 
   if (USE_MIX == 1)
     fputs("-----Final Hop-----\n", outfile);
   fputs(to, outfile);
-  if (to[strlen(to) - 1] != '\n') {
+  if (strlen(to) > 0 && to[strlen(to) - 1] != '\n') {
     if (USE_SYSLOG)
       syslog(LOG_WARNING, "Truncated destination buffer");
     fputs("\n", outfile);
@@ -325,6 +330,7 @@ remove_headers(const char *infilename, const char *outfilename, const char *to)
 	headers = 0;
     }
   }
-  fclose(infile);
   fclose(outfile);
+  fclose(infile);
+  unlink(infilename);
 }
